Package: easyPSID
Title: Reading, Formatting, and Organizing the Panel Study of Income
        Dynamics (PSID)
Version: 0.1.3
Authors@R: person("Brian", "Aronson",email = "bdaronson@gmail.com", role = c("aut", "cre"))
Description: Provides various functions for reading and preparing the Panel Study of Income Dynamics (PSID) for longitudinal analysis, including functions that read the PSID's fixed width format files directly into R, rename all of the PSID's longitudinal variables so that recurring variables have consistent names across years, simplify assembling longitudinal datasets from cross sections of the PSID Family Files, and export the resulting PSID files into file formats common among other statistical programming languages ('SAS', 'STATA', and 'SPSS').
Depends: R (>= 3.0.1), stringr (>= 1.0.0), LaF (>= 0.6.0), foreign (>=
        0.8-67)
License: MIT + file LICENSE
Encoding: UTF-8
Suggests: readxl (>= 1.4.0), testthat (>= 3.0.0)
Config/testthat/edition: 3
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-08-04 14:54:57 UTC; bdaro
Author: Brian Aronson [aut, cre]
Maintainer: Brian Aronson <bdaronson@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-04 17:30:07 UTC
Built: R 4.6.0; ; 2026-08-04 19:36:11 UTC; unix

# Generated by roxygen2: do not edit by hand

export(convert_from_rds)
export(convert_to_rds)
export(create_custom_panel)
export(create_extract)
export(find_description)
export(find_name)
export(find_years)
export(rename_fam_vars)
export(rename_ind_vars)
export(unzip_all_files)
importFrom(LaF,laf_open_fwf)
importFrom(foreign,write.dta)
importFrom(foreign,write.foreign)
importFrom(stringr,str_extract)
importFrom(stringr,str_match)
importFrom(stringr,str_match_all)
importFrom(utils,object.size)
importFrom(utils,unzip)

# easyPSID README.md

## Overview

The easyPSID package is designed to simplify the task of reading the Panel Study of Income Dynamics (PSID) into R and preparing the data for analysis. 

Although the PSID is one of the most comprehensive longitudinal datasets for examining health and life course patterns among American families, preparing the PSID for analysis can be a difficult task. The PSID comes with no scripts for being read into R, and every wave of each PSID longitudinal variable has a unique name. For example, variables in the 1968 PSID Family File are named “V1” through “V440” whereas variables within the 1969 PSID Family File are named “V441” through “V1017,” even though of the 1969 Family File variables match those in the 1968 Family File. These aspects of the PSID can pose difficulties for R users who want to conduct longitudinal analyses in the PSID. Fortunately, the easyPSID package is designed to make working with the PSID as easy as possible.

## Example

After users have downloaded several waves of the PSID Packaged Data Family Files from https://simba.isr.umich.edu/, a common first use of the easyPSID package is to unzip each wave of the family files, convert them to .rds format, rename all longitudinal variables to consistent names across years, and save these resulting renamed datasets to .rds format with the following code (after amending for directory names). For example:

    library(easyPSID)
    unzip_all_files(
        in_direc="C:/PSID/Zip Files",
        out_direc="C:/PSID/Unzipped Files",
    )
    convert_to_rds(
        in_direc="C:/PSID/Unzipped Files",
        out_direc="C:/PSID/rds Files",
    )
    rename_fam_vars(
        in_direc="C:/PSID/rds Files",
        out_direc="C:/PSID/renamed Files",
    )

## Installation

This package can be directly installed via CRAN with:
    install.packages("easyPSID")
    
Alternatively, newest versions of this package can be installed with:
    devtools::install_github("BrianAronson/easyPSID")

However, prior to working with the easyPSID package, users will need to have already downloaded packaged PSID data for the years that they wish to work with from https://simba.isr.umich.edu/. PSID's packaged data can only be accessed by loading this URL and navigating through the following tabs: Data=> Packaged Data=> Main and Supplemental Studies. The current functions only work with the PSID's family files and cross-sectional individual file. For reference, once downloaded, the family files typically have a naming structure like "fam1968.zip", "fam1969.zip", etc. and the individual cross section will be named something like "ind2015er.zip".

##Function Overview

More detailed vignettes for this package are in development, but below is a brief outline of each function in this package:

- **unzip\_all\_files**: Unzips all files in the supplie directory.
- **convert\_to\_rds**: Read PSID fixed width files into R and saves output in .rds format.
- **rename\_fam\_vars**: Renames all variables in longitudinal family files to their names when they were first available in the dataset.
- **rename\_ind\_vars**: Renames all variables in longitudinal family files to their names when they were first available in the dataset.
- **create\_custom\_panel**: Creates a longitudinal dataset with provided family files consisting of a custom set of  variables selected by the user.
- **create\_extract**: Creates a longitudinal dataset with provided family files consisting of the dataset's N most commonly reoccurring variables.
- **convert\_from\_rdata**: Converts PSID .rds files to files compatible with STAT, SAS, or SPSS.
- **find\_name**: Returns the new name (via the rename\_fam\_vars and rename\_ind\_vars functions) of a specified variable.
- **find\_years**: Returns each year that a given variable is available in the PSID, and its names during those years.
- **find\_description**: Returns the description of a given variable in the PSID with the variable labels provided by the PSID.

# easyPSID

## Overview

[CRAN package](https://CRAN.R-project.org/package=easyPSID) designed to simplify the task of reading the Panel Study of Income Dynamics (PSID) into R and preparing the data for analysis. Although the PSID is one of the most comprehensive longitudinal datasets for examining health and life course patterns among American families, preparing the PSID for analysis can be a difficult task. The PSID comes with no scripts for being read into R, and every wave of each PSID longitudinal variable has a unique name. For example, variables in the 1968 PSID Family File are named `V1-V440` whereas variables within the 1969 PSID Family File are named `V441-V1017`, even though most variables in the 1969 Family Files match those in the 1968 Family File. These aspects of the PSID pose difficulties for R users who want to conduct longitudinal analyses in the PSID. easyPSID simplifies these tasks and several others.

## Example

After downloading multiple waves of the PSID Packaged Data Family Files from https://simba.isr.umich.edu/, a common use of easyPSID is to unzip each wave of the family files, convert them to .rds format, rename all longitudinal variables to consistent names across years, and save the resulting renamed datasets to .rds format. The development version's cross-year reference currently covers the 1968-2023 waves. This can be done as follows (assuming the PSID files were saved into `C:/PSID/Zip Files`):

    library(easyPSID)
    unzip_all_files(
        in_direc="C:/PSID/Zip Files",
        out_direc="C:/PSID/Unzipped Files"
    )
    convert_to_rds(
        in_direc="C:/PSID/Unzipped Files",
        out_direc="C:/PSID/rds Files"
    )
    rename_fam_vars(
        in_direc="C:/PSID/rds Files",
        out_direc="C:/PSID/renamed Files"
    )

`convert_to_rds()` expects the fixed-width ASCII `.txt` file and matching Stata
`.do` definition file included in each PSID packaged-data ZIP. It does not read
Stata `.dta` files. If you already have a `.dta` file, import it with a Stata
reader such as `haven::read_dta()`, preserve the original PSID variable names,
and save the resulting data frame with `saveRDS()` before calling a renaming
function.

## Installation

This package can be directly installed via CRAN with `install.packages("easyPSID")`. Alternatively, newest versions of this package can be installed with `devtools::install_github("BrianAronson/easyPSID")`

Prior to working with the easyPSID package, users will need to download packaged PSID data for the family files and cross-sectional individual file from https://simba.isr.umich.edu/.

The small files bundled under `inst/extdata` are excerpts used to exercise the
package examples. Users must register with PSID and acquire full public-release
data directly from PSID.

## Function Overview

Below is a brief outline of each function in this package:

- **unzip\_all\_files**
    - Unzips all files in the supplied directory.
- **convert\_to\_rds**
    - Read PSID fixed width files into R and saves output in .rds format.
- **rename\_fam\_vars**
    - Renames all variables in longitudinal family files to their names when they were first available in the dataset.
- **rename\_ind\_vars**
    - Reshapes the cross-year Individual File to long format and gives recurring variables their earliest names while retaining `ER30001` and `ER30002` as stable person identifiers.
- **create\_custom\_panel**
    - Creates a longitudinal dataset with provided family files consisting of a custom set of  variables selected by the user.
- **create\_extract**
    - Creates a longitudinal dataset with provided family files consisting of the dataset's N most commonly reoccurring variables.
- **convert\_from\_rdata**
    - Converts PSID .rds files to files compatible with STAT, SAS, or SPSS.
- **find\_name**
    - Returns the new name (via the rename\_fam\_vars and rename\_ind\_vars functions) of a specified variable.
- **find\_years**
    - Returns each year that a given variable is available in the PSID, and its names during those years.
- **find\_description**
    - Returns the description of a given variable in the PSID with the variable labels provided by the PSID.

#' Export PSID files to another statistical language
#' @description Exports all .rds files in the chosen directory into a common file format used by one of three other statistical programming languages (SPSS, SAS, and STATA). Unlike most alternatives, this function retains all variable labels provided by the PSID.
#' @param language Language to export PSID .rds files into (options include SPSS, SAS, and STATA)
#' @param in_direc Directory of PSID .rds files to export. Note that large files can take a long time to export.
#' @param out_direc Directory for exported files to be placed
#' @keywords PSID
#' @export 
#' @importFrom foreign write.dta write.foreign
#' @examples 
#' convert_from_rds(
#'     language="STATA", 
#'     in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
#'     out_direc=tempdir()
#' )

convert_from_rds<-function(language, in_direc, out_direc){
    #1) Prepare directories and file names
        # in_direc<-ifelse(is.null(in_direc),getwd(),in_direc)
        # out_direc<-ifelse(is.null(out_direc),in_direc,out_direc)
        dir.create(out_direc,showWarnings = F)
        filenames<-list.files(path=in_direc,pattern= c(".*\\.rds$" )) #only files ending with ".rds"
    #2) Export data
      for(i in 1:length(filenames)){  
        tempdataframename<-readRDS(paste(in_direc,"/",filenames[i],sep=""))
        if(language=="SPSS"){
          write.foreign(tempdataframename, paste(out_direc,"/",filenames[i],".txt",sep=""), paste(out_direc,"/",tempdataframename,".sps",sep=""),   package="SPSS")
          }
        if(language=="SAS"){
          write.foreign(tempdataframename, paste(out_direc,"/",filenames[i],".csv",sep=""), paste(out_direc,"/",tempdataframename,".sas",sep=""), dataname=gsub(" ","",tempdataframename),  package="SAS")
          }
        if(language=="STATA"){
          write.dta(tempdataframename,  paste(out_direc,"/",filenames[i],".dta",sep=""))
          }
        message(paste(filenames[i],"Exported"))
      }
}

#' Find years of PSID variable
#' @description Finds the years and corresponding variable names for any longitudinal PSID variable in the PSID Family Files and Individual file.
#' @param variable Variable name to look up
#' @param var_names Report names for longitudinal PSID variable across years (default=TRUE)
#' @keywords PSID
#' @export
#' @examples find_years(variable="V3",var_names=FALSE)


find_years <- function(variable, var_names = TRUE) {
    variable <- as.character(variable)
    years <- .find_reference_years(variable)

    if (is.null(years)) {
        return(variable)
    }
    if (isTRUE(var_names)) {
        return(years)
    }
    years$Year
}

test_that("current family variables resolve across waves", {
    release_numbers <- find_years("ER78001")

    expect_equal(tail(release_numbers$Year, 3L), c(2019, 2021, 2023))
    expect_equal(
        tail(release_numbers$Name, 3L),
        c("ER72001", "ER78001", "ER82001")
    )
    expect_equal(find_name("ER82001", var_year = FALSE), "V21601")
})

test_that("single-wave variables do not cause lookup errors", {
    person_number <- find_years("ER30002")

    expect_equal(person_number$Year, 1968)
    expect_equal(person_number$Name, "ER30002")
    expect_equal(find_name("ER30002", var_year = FALSE), "ER30002")
})

test_that("unknown variables retain their original names", {
    expect_equal(find_years("NOT_A_PSID_VARIABLE"), "NOT_A_PSID_VARIABLE")
    expect_equal(find_name("NOT_A_PSID_VARIABLE"), "NOT_A_PSID_VARIABLE")
})

test_that("lookup functions reject multiple variables", {
    expect_error(
        find_years(c("ER78001", "ER82001")),
        "exactly one PSID variable"
    )
})

#' Rename longitudinal Family File variables
#' @description Renames all longitudinal variables in every PSID Family File of a given directory, such that variables are labeled with the variable name used when the variable was first made available in the PSID. For example, the "Release Number" variable was first recorded in the PSID dataset in 1968 as variable "V1" but its name in the 1969 family file is "V441". This program changes the "Release Number" variable name to "V1" in 1968 and all subsequent waves.
#' @param in_direc Directory of PSID .rds files to rename
#' @param out_direc Directory for renamed PSID .rds files to be saves to. Warning: If no directory specified, this function will overwrite the Family Files in the current directory.
#' @keywords PSID
#' @export
#' @examples
#' rename_fam_vars(
#'     in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
#'     out_direc=tempdir()
#' )

rename_fam_vars <- function(in_direc, out_direc) {
    file_names <- list.files(
        path = in_direc,
        pattern = "^FAM.*\\.rds$",
        full.names = TRUE
    )
    if (length(file_names) == 0L) {
        stop("No Family File .rds files were found in `in_direc`.", call. = FALSE)
    }

    dir.create(out_direc, recursive = TRUE, showWarnings = FALSE)
    supported_years <- sub("^Y", "", names(data$psid_all))

    for (file_name in file_names) {
        short_name <- basename(file_name)
        file_year <- gsub("\\D+", "", short_name)
        year_column <- match(file_year, supported_years)
        if (is.na(year_column)) {
            stop(
                paste0(
                    "The year in `", short_name, "` is not present in the internal ",
                    "PSID index. Supported years are ",
                    paste(supported_years, collapse = ", "), "."
                ),
                call. = FALSE
            )
        }

        family_data <- readRDS(file_name)
        renamed_variables <- names(family_data)

        for (variable_index in seq_along(renamed_variables)) {
            matching_rows <- which(
                data$psid_all[[year_column]] == renamed_variables[variable_index]
            )
            if (length(matching_rows) == 0L) {
                next
            }

            names_by_year <- unlist(
                data$psid_all[matching_rows[1L], , drop = FALSE],
                use.names = FALSE
            )
            names_by_year <- names_by_year[!is.na(names_by_year) & nzchar(names_by_year)]
            renamed_variables[variable_index] <- names_by_year[1L]
        }

        names(family_data) <- renamed_variables
        saveRDS(family_data, file = file.path(out_direc, short_name))
        message(short_name, " renamed")
    }
}

test_that("family variables are renamed through the 2023 index", {
    input_directory <- file.path(tempdir(), "easypsid-family-input")
    output_directory <- file.path(tempdir(), "easypsid-family-output")
    dir.create(input_directory, recursive = TRUE, showWarnings = FALSE)

    family_data <- data.frame(
        ER78001 = 1,
        ER78002 = 100,
        project_specific = 3
    )
    saveRDS(family_data, file.path(input_directory, "FAM2021.rds"))

    expect_message(
        rename_fam_vars(input_directory, output_directory),
        "FAM2021.rds renamed"
    )
    renamed <- readRDS(file.path(output_directory, "FAM2021.rds"))

    expect_equal(names(renamed), c("V21601", "V3", "project_specific"))
})

test_that("family renaming reports missing and unsupported inputs", {
    empty_directory <- file.path(tempdir(), "easypsid-empty-family-input")
    dir.create(empty_directory, recursive = TRUE, showWarnings = FALSE)

    expect_error(
        rename_fam_vars(empty_directory, tempdir()),
        "No Family File"
    )

    future_directory <- file.path(tempdir(), "easypsid-future-family-input")
    dir.create(future_directory, recursive = TRUE, showWarnings = FALSE)
    saveRDS(data.frame(ER86001 = 1), file.path(future_directory, "FAM2025.rds"))

    expect_error(
        rename_fam_vars(future_directory, tempdir()),
        "not present in the internal PSID index"
    )
})

#' Find description of PSID variable
#' @description Finds the descriptions of selected PSID variables.
#' @param variables Variable names to look up (as individual string or vector of strings)
#' @keywords PSID
#' @export
#' @examples find_description(variables=c("V2","V30"))

find_description<-function(variables){
    #1) Make sure variable is in character format
        variables<-as.character(variables)
    #2) Find variables in reference dataset
        varrow<-data$psid_desc[match(variables,data$psid_desc[,1]),]
    #3) If variables is not in reference dataset, just show the variables name
        if(all(is.na(varrow))){
          return(variables)
        }
    #4) Otherwise, print variables name and description
        return(varrow)
}

test_that("the internal public index reaches the 2023 wave", {
    reference_data <- get("data", envir = asNamespace("easyPSID"))

    expect_equal(tail(names(reference_data$psid_all), 3L), c("Y2019", "Y2021", "Y2023"))
    expect_true("ER78001" %in% reference_data$psid_all$Y2021)
    expect_true("ER82001" %in% reference_data$psid_all$Y2023)
})

test_that("stable person identifiers are retained in every wave", {
    reference_data <- get("data", envir = asNamespace("easyPSID"))

    for (variable in c("ER30001", "ER30002")) {
        identifier_row <- which(
            reference_data$psid_ind == variable,
            arr.ind = TRUE
        )[1L, "row"]
        expect_true(all(reference_data$psid_ind[identifier_row, ] == variable))
    }
})

test_that("new variables have descriptions", {
    description <- find_description("ER82001")

    expect_equal(description$Variable, "ER82001")
    expect_false(is.na(description$Description))
    expect_true(nzchar(description$Description))
})

#' @details

#' This package is designed for use with the PSID's packaged data Family Files, available at https://simba.isr.umich.edu/. See easyPSID's readme at https://github.com/BrianAronson/easyPSID/blob/master/README.md for a more detailed overview and guide for using this package.
#'
"_PACKAGE"

#' Renames longitudinal Cross-year Individual variables and saves in long format
#' @description Renames all repeated variables in the Cross-year Individual file so that matching variables across waves have the same name, and transforms the resulting dataset into long format. The longitudinal file does not include rows for respondents who were missing in a given wave, and cross-sectional variables are marked as NA during waves when they were not asked. In addition, the resulting file adds two variables for ease of use: "Year" and "fam_id_68". 
#' 
#' This function may require up to 8gb of RAM, and will likely throw "cannot allocate memory" errors to users with less RAM on their computer. Users with memory issues should implement the "only_long_vars" or "cust_vars" options.
#'
#' @param in_direc Directory of PSID Cross-year Individual file .rds file
#' @param out_direc Directory for renamed and transformed PSID Cross-year Individual file to be saved to
#' @param only_long_vars Keep only longitudinal variables in dataset
#' @param cust_vars Custom variables to keep in dataset (as character vector). Output will always include "ER30001", "ER30002", "fam_id_68", and "Year"
#' @keywords PSID
#' @export
#' @examples 
#' rename_ind_vars(
#'    only_long_vars=TRUE, 
#'    in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
#'    out_direc=tempdir()
#' )

rename_ind_vars<-function(in_direc,out_direc,only_long_vars=F,cust_vars=NULL){
    #1) Deprecated - Set directory
        # in_direc<-ifelse(is.null(in_direc),getwd(),in_direc)
        # out_direc<-ifelse(is.null(out_direc),getwd(),out_direc)
    #2) Load rds file
        temp_files<-list.files(path=in_direc,pattern= c("^IND.*\\.rds$" ),full.names=T)
        if(length(temp_files)==0L){
          stop("No cross-year Individual File .rds file was found in `in_direc`.",call.=FALSE)
        }
        temp_file<-temp_files[1]
        temp_file2<-basename(temp_file)
        df<-readRDS(temp_file)
    #3) Identify dimensions of eventual dataframe
        ncols<-dim(data$psid_ind)[1]
        nyears<-dim(data$psid_ind)[2]
        ninds<-nrow(df)
        nrows<-nyears*ninds
    #4) Create transposed version of data$psid_ind, sorted by time first in dataset
        early_name<-apply(data$psid_ind,1,function(x) x[x!=""][1])
        index<-data$psid_ind[order(early_name),]
        index<-t(index)
    #5) For convenience, create a column in data that is all NAs
        df$colna<-NA
    #6) Identify whether variable is cross-sectional
        cross<-apply(index,2, function(x) length(x[x!=""]))
        cross<-ifelse(cross==1,1,0)
    #7) Identify whether observation was not present during a given wave
        #ER30003 indicates whether non-response in given wave
            response_matches<-which(index=="ER30003",arr.ind = T)
            ER30003<-response_matches[1,"col"]
            #i) Find corresponding variable names
                varnames<-as.vector(index[,ER30003])
            #ii) Replace missing variable names with colna
                varnames[varnames=="" | !varnames %in% names(df)]<-"colna"
            #iii) Pull out those column names and convert to vector
                resp<-vector()
                for(j in 1:length(varnames)){
                  resp<-c(resp,df[,varnames[j]])
                }
            #iv) Remove missing individuals
                resp[is.na(resp)]<-0
                resp[resp>0]<-1
    #8) Subset to custom variables or longitudinal variables
        if(!is.null(cust_vars)){
          required_ids<-c("ER30001","ER30002")
          cust_vars<-unique(c(cust_vars,required_ids))
          indcols<-vapply(cust_vars,function(x){
            matches<-which(index==x,arr.ind=TRUE)
            if(nrow(matches)==0L){
              stop(paste0("Variable `",x,"` was not found in the Individual File index."),call.=FALSE)
            }
            matches[1,"col"]
          },integer(1))
          index<-index[,indcols]
        }else if(only_long_vars==T){
          index<-index[,cross==0]
        }
    #9) For each unique variable in individual file, convert to long format
        #i) Find corresponding variable names across waves
            ind_longdf<-lapply(as.data.frame(index),as.vector)
        #ii) Replace missing variable names with colna
            ind_longdf<-lapply(ind_longdf,function(x){
              ifelse(x=="" | !x %in% names(df),"colna",x)
            })
        #iii) Pull out those columns
            ind_longdf<-lapply(ind_longdf,function(x) df[,x])
        #iv) Convert columns to single vector
            fastunlist<-function(x){
              x<-c(as.matrix(x))
              x<-x[resp==1] #(not usually part of function)
            }
            percent <- function(x, digits = 0, format = "f", ...) {
              paste0(formatC(100 * x, format = format, digits = digits, ...), "%")
            }
            for(i in 1:length(ind_longdf)){
              ind_longdf[[i]]<-fastunlist(ind_longdf[[i]])
              if(i/10==floor(i/10)){
                gc()
                message(percent(i/length(ind_longdf)))
              }
            }
            gc()
        #v) Convert list to dataframe
            ind_longdf<-do.call(cbind,ind_longdf)
            gc()
            ind_longdf<-as.data.frame(ind_longdf)
            gc()
    #10) Add metadata to dataframe
        #a) Create proper names for columns
            if(!is.null(cust_vars)){
              early_name2<-cust_vars
            }else if(only_long_vars==T){
              early_name2<-apply(index,2,function(x) x[x!=""][1])
            }else{
              early_name2<-early_name[order(early_name)]  
            }
            names(ind_longdf)<-early_name2
        #b) Bring back variable descriptions
            temp<-find_description(names(ind_longdf))
            attr(ind_longdf,"var.labels")<-temp$Description
        #c) Add a year variable
            uyears<-gsub('\\D+','', row.names(index))
            Year<-rep(uyears,each=ninds)
            Year<-Year[resp==1]
            ind_longdf$Year<-Year
        #d) Add a family id variable
            ind_longdf$fam_id_68<-ind_longdf$ER30001
      #11) Save file
          dir.create(out_direc,showWarnings = F)
          temp_file2<-gsub(".rds","",temp_file2)
          saveRDS(ind_longdf,file=paste(out_direc,"/",temp_file2," - Long Format.rds",sep=""))
          message(percent(1))
}

test_that("individual reshaping preserves stable person identifiers", {
    reference_data <- get("data", envir = asNamespace("easyPSID"))
    relationship_row <- which(
        reference_data$psid_ind == "ER30003",
        arr.ind = TRUE
    )[1L, "row"]
    response_variables <- unique(unlist(
        reference_data$psid_ind[relationship_row, , drop = FALSE],
        use.names = FALSE
    ))
    response_variables <- response_variables[nzchar(response_variables)]

    individual_data <- as.data.frame(
        setNames(rep(list(1), length(response_variables)), response_variables),
        check.names = FALSE
    )
    individual_data$ER30001 <- 123
    individual_data$ER30002 <- 4

    input_directory <- file.path(tempdir(), "easypsid-individual-input")
    output_directory <- file.path(tempdir(), "easypsid-individual-output")
    dir.create(input_directory, recursive = TRUE, showWarnings = FALSE)
    saveRDS(individual_data, file.path(input_directory, "IND2023.rds"))

    expect_message(
        rename_ind_vars(
            in_direc = input_directory,
            out_direc = output_directory,
            cust_vars = "ER30002"
        ),
        "100%"
    )

    reshaped <- readRDS(file.path(
        output_directory,
        "IND2023 - Long Format.rds"
    ))
    expect_true(all(reshaped$ER30001 == 123))
    expect_true(all(reshaped$ER30002 == 4))
    expect_equal(reshaped$fam_id_68, reshaped$ER30001)
})

#' Convert all PSID files from .txt format to .rds format
#' @description Converts all PSID fixed width format .txt files in a selected directory into .rds format. This function expects the ASCII .txt file and matching Stata .do definition file contained in each PSID packaged-data .zip download. It does not read Stata .dta files. All files must retain the names they had when first unzipped.
#' @param in_direc Directory containing unzipped PSID .txt and .do files
#' @param out_direc Directory to place PSID .rds files into
#' @keywords PSID
#' @export
#' @importFrom stringr str_extract str_match str_match_all
#' @importFrom LaF laf_open_fwf
#' @examples
#' convert_to_rds(
#'     in_direc=system.file("extdata","unzip_dir", package = "easyPSID"),
#'     out_direc=tempdir()
#' )

convert_to_rds <- function(in_direc, out_direc) {
    txt_files <- list.files(
        path = in_direc,
        pattern = "\\.txt$",
        full.names = TRUE,
        ignore.case = TRUE
    )
    if (length(txt_files) == 0L) {
        stop("No fixed-width .txt files were found in `in_direc`.", call. = FALSE)
    }

    do_files <- list.files(
        path = in_direc,
        pattern = "\\.do$",
        full.names = TRUE,
        ignore.case = TRUE
    )
    do_stems <- tolower(tools::file_path_sans_ext(basename(do_files)))
    dir.create(out_direc, recursive = TRUE, showWarnings = FALSE)

    for (txt_file in txt_files) {
        txt_name <- basename(txt_file)
        txt_stem <- tools::file_path_sans_ext(txt_name)
        matching_do_files <- do_files[do_stems == tolower(txt_stem)]
        if (length(matching_do_files) != 1L) {
            stop(
                paste0(
                    "Expected exactly one matching .do file for `", txt_name,
                    "`, but found ", length(matching_do_files), "."
                ),
                call. = FALSE
            )
        }

        stata_code <- readLines(matching_do_files, warn = FALSE)
        infix_line <- grep("^\\s*infix\\b", stata_code, ignore.case = TRUE)[1L]
        using_lines <- grep("^\\s*using\\b", stata_code, ignore.case = TRUE)
        using_line <- using_lines[using_lines > infix_line][1L]
        if (is.na(infix_line) || is.na(using_line)) {
            stop(
                paste0("Could not locate the infix definition in `", basename(matching_do_files), "`."),
                call. = FALSE
            )
        }

        infix_code <- paste(stata_code[(infix_line + 1L):(using_line - 1L)], collapse = " ")
        field_pattern <- paste0(
            "(?:\\b(?:byte|int|long|float|double|str[0-9]+)\\s+)?",
            "([A-Za-z][A-Za-z0-9_]*)\\s+([0-9]+)\\s*-\\s*([0-9]+)"
        )
        fields <- str_match_all(infix_code, field_pattern)[[1L]]
        if (nrow(fields) == 0L) {
            stop(
                paste0("No fixed-width fields were found in `", basename(matching_do_files), "`."),
                call. = FALSE
            )
        }

        variable_names <- fields[, 2L]
        columns <- as.integer(fields[, 4L]) - as.integer(fields[, 3L]) + 1L

        label_pattern <- paste0(
            "^\\s*label\\s+variable\\s+([A-Za-z][A-Za-z0-9_]*)\\s+",
            "\"(.*)\"\\s*;?\\s*$"
        )
        label_fields <- str_match(stata_code, label_pattern)
        label_fields <- label_fields[!is.na(label_fields[, 2L]), , drop = FALSE]
        labels_by_name <- stats::setNames(
            trimws(label_fields[, 3L]),
            label_fields[, 2L]
        )
        variable_labels <- unname(labels_by_name[variable_names])

        converted_data <- laf_open_fwf(
            txt_file,
            column_widths = columns,
            column_types = rep("double", length(columns))
        )
        converted_data <- converted_data[, ]
        names(converted_data) <- variable_names
        attr(converted_data, "var.labels") <- variable_labels

        file_year <- str_extract(txt_name, "[0-9]+")
        file_type <- toupper(substr(txt_name, 1L, 3L))
        output_name <- paste0(file_type, file_year, ".rds")
        saveRDS(converted_data, file = file.path(out_direc, output_name))
        message(output_name, " converted")
    }
}

test_that("conversion pairs data with its import script and ignores format scripts", {
    input_directory <- file.path(tempdir(), "easypsid-convert-input")
    output_directory <- file.path(tempdir(), "easypsid-convert-output")
    dir.create(input_directory, recursive = TRUE, showWarnings = FALSE)
    dir.create(output_directory, recursive = TRUE, showWarnings = FALSE)

    writeLines(
        c(
            "infix",
            "    ER82001 1 - 1    long ER82002 2 - 6",
            "using FAM2023ER.txt",
            "label variable ER82001 \"Release number\"",
            "label variable ER82002 \"Family interview number\""
        ),
        file.path(input_directory, "FAM2023ER.do")
    )
    writeLines(
        "label define ER82001L 1 \"Release one\"",
        file.path(input_directory, "FAM2023ER_formats.do")
    )
    writeLines(c("100001", "200002"), file.path(input_directory, "FAM2023ER.txt"))

    convert_to_rds(input_directory, output_directory)
    converted_data <- readRDS(file.path(output_directory, "FAM2023.rds"))

    expect_equal(names(converted_data), c("ER82001", "ER82002"))
    expect_equal(nrow(converted_data), 2L)
    expect_equal(
        attr(converted_data, "var.labels"),
        c("Release number", "Family interview number")
    )
})

test_that("conversion reports a missing matching import script", {
    input_directory <- file.path(tempdir(), "easypsid-missing-do-input")
    output_directory <- file.path(tempdir(), "easypsid-missing-do-output")
    dir.create(input_directory, recursive = TRUE, showWarnings = FALSE)
    writeLines("100001", file.path(input_directory, "FAM2023ER.txt"))

    expect_error(
        convert_to_rds(input_directory, output_directory),
        "matching .do file"
    )
})

#' Lookup new longitudinal variables names
#' @description Finds the new name of any longitudinal variable in the PSID Family Files or Individual files following implementation of the rename_fam_vars and rename_ind_vars functions.
#' @param variable Variable to look up
#' @param var_year Report year when renamed variable first entered the PSID dataset (default=TRUE)
#' @keywords PSID
#' @export
#' @examples find_name(variable="V1244",var_year=FALSE)


find_name <- function(variable, var_year = TRUE) {
    variable <- as.character(variable)
    years <- .find_reference_years(variable)

    if (is.null(years)) {
        return(variable)
    }
    if (isTRUE(var_year)) {
        return(years[1L, , drop = FALSE])
    }
    years$Name[1L]
}

.find_reference_years <- function(variable) {
    variable <- as.character(variable)
    if (length(variable) != 1L) {
        stop("`variable` must contain exactly one PSID variable name.", call. = FALSE)
    }

    matches <- which(data$psid_all == variable, arr.ind = TRUE)
    if (nrow(matches) == 0L) {
        return(NULL)
    }

    reference_row <- data$psid_all[matches[1L, "row"], , drop = FALSE]
    names_by_year <- unlist(reference_row, use.names = TRUE)
    names_by_year <- names_by_year[!is.na(names_by_year) & nzchar(names_by_year)]

    data.frame(
        Year = as.integer(sub("^Y", "", names(names_by_year))),
        Name = unname(names_by_year),
        stringsAsFactors = FALSE,
        row.names = NULL
    )
}

#' Create subset of common family file variables in long format
#' @description Creates an extract dataset in long format consisting of the 500 most frequently reoccurring PSID Family Variables across all supplied waves of the PSID.
#' @param in_direc Directory containing waves of the Family Files in .rds format
#' @param out_direc Directory to place export file into
#' @param num_vars Number of variables to include in export dataset (default = 500). High variable counts with many waves of data require a significant amount of RAM, and may cause this function to throw errors if a computer's RAM is insufficient
#' @param all_years Select most common variables based on all years of the PSID rather than based in the data actually supplied
#' @keywords PSID
#' @export
#' @examples 
#' create_extract(
#'     in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
#'     out_direc=tempdir(),
#'     num_vars=25,
#' )

create_extract<-function(in_direc, out_direc, num_vars=500, all_years=F){
    #1) Assign directories
        # in_direc<-ifelse(is.null(in_direc),getwd(),in_direc)
        # out_direc<-ifelse(is.null(out_direc),getwd(),out_direc)
        filenames<-list.files(path=in_direc,pattern= c("^FAM.*\\.rds$" ),full.names = T)
        filenames2<-list.files(path=in_direc,pattern= c("^FAM.*\\.rds$" ))
    #1.5) Error check
        #Don't run if only one year provided
            if(length(filenames)<2){
                filenames<-paste("Less than two waves of data found in in_direc. Two or more waves of data required")
                stop(message)
            }
    #2) Determine variable names
        #a) determine years of supplied data
            data_years<-gsub('\\D+','',filenames2)
        #b) pull psid info
            tdf<-data$psid_fam
        #c) only include matching years
            if(all_years==F){
                names(tdf)<-gsub('\\D+','',names(tdf))
                tdf<-tdf[,names(tdf) %in% data_years]
            }
        #d) find most common variables
            if(ncol(as.data.frame(tdf))>1){
                var_freq<-apply(tdf,1,function(x) sum(x!=""))
            }else{
                var_freq<-sum(tdf[,1]!="")
            }
            var_rank<-rank(var_freq,ties.method="first")
            var_rank<-max(var_rank)-var_rank
            var_common<-tdf[var_rank<=num_vars,]
            var_names<-as.vector(apply(var_common,1,function(x) x[x!=""][1]))
            
    #3) Redundant steps to make remainder of formula work.
        temp_names<-var_names
        #a) Identify row and column of each variable
            index<-lapply(temp_names,function(x) which(data$psid_all==x,arr.ind=T))
        #b) Extract rows from index
            row<-sapply(index,function(x) x[1])
        #c) Keep index of current variables
            new_ind<-data$psid_all[row,]
        #d) Rename columms
            names(new_ind)<-gsub('\\D+','', names(new_ind))
        #e) Identify earliest name of each variable
            early_name<-as.vector(apply(new_ind,1,function(x) x[x!=""][1]))

    #4) Create a list of longitudinal datasets with variables selected above
        temp_datal<-list()
        for(j in 1:length(filenames)){
        #a) Load first file and rename to "temp_data"
            temp_data<-readRDS(filenames[j])
        #b) Create vector of variable names
            temp_names<-names(temp_data)
        #c) Only keep columns that match possible variable names
            #i) Identify dataframe year
                temp_year<-gsub('\\D+','', filenames2[j])
            #ii) Identify matchin column in dataset
                column<-match(temp_year,names(new_ind))
            #iii) Find variable names of requested variables as of a given year
                keep_vars<-match(new_ind[,column],temp_names)
                # In case variables already renamed, try above with earliest variable names
                    keep_varsalt<-match(early_name,temp_names)
                    if(sum(is.na(keep_varsalt))<sum(is.na(keep_vars))){
                      keep_vars<-keep_varsalt
                    }
            #iv) Pull those variables and replace no matches with NAs
                keep_vars2<-keep_vars
                keep_vars2[is.na(keep_vars)]<-1
                temp_data2<-temp_data[,keep_vars2]
                temp_data2[,is.na(keep_vars)]<-NA
            #v) rename variables, add year, and append to list
                names(temp_data2)<-var_names
                temp_data2$Year<-as.numeric(temp_year)
                temp_datal[[j]]<-temp_data2
                message(paste(filenames2[j],"Added to Panel"))
        }

    #5) Put above datasets to dataframe format and save
        PSIDPanel<-do.call(rbind,temp_datal)
        dir.create(out_direc,showWarnings = F)
        saveRDS(PSIDPanel,file=paste(out_direc,"/","PSID Extract.rds",sep = ""))
}

#' Unzip all PSID files
#' @description Unzips all .zip_files files in the specified directory.
#' @param in_direc Directory of .zip files to be unzipped
#' @param out_direc Directory for unzipped PSID files to be placed
#' @keywords PSID
#' @export
#' @importFrom utils object.size unzip
#' @examples
#' unzip_all_files(
#'     in_direc=system.file("extdata", "zip_dir", package = "easyPSID"),
#'     out_direc=tempdir()
#' )

unzip_all_files<-function(in_direc,out_direc){
    #1) Deprecated - Set directories to current directory if none supplied
        # in_direc<-ifelse(is.null(in_direc),getwd(),in_direc)
        # out_direc<-ifelse(is.null(out_direc),in_direc,out_direc)
    #2) Find all zip_files files in folder
        zip_files<-list.files(path=in_direc,pattern= c("*\\.zip$" ),full.names=T)
    #3) Unzip_files files to specified directories
        dir.create(out_direc,showWarnings = F)
        for(i in 1:length(zip_files)){
          unzip(zip_files[i],exdir=out_direc)
        }
    #4) Indicate that files are unzip_filesped
        message("Files Unzipped")
}

#' Create custom longitudinal (panel) dataset with PSID Family Files
#' @description Uses the longitudinal PSID Family Files to create a custom longitudinal dataset in long format based on all PSID .rds Family files in a selected directory.
#' 
#' This function can work with data that has been renamed via the rename_fam_vars function or data just converted to .rds format via the convert_to_rds function. It will creates NAs for years when a given variable was not available, and creates a new variable ("Year") to specify the panel of data included in the custom dataset. If a provided variable exists in other waves of the family files under a different name, all waves of that variable will be included in the resulting dataset.
#' 
#' To create a longitudinal family file of the PSID with all variables in the PSID Family Files, it is recommended that one uses the create_extract function instead. However, such a file can be very large when using many waves of the PSID. Users with more than five waves of the PSID Family Files are highly recommended to avoid creating a longitudinal dataset with all unique Family File variables.
#' 
#' @param var_names Variable names to include in custom longitudinal dataset (as vector of strings)
#' @param in_direc Directory of PSID .rds to use for custom longitudinal dataset
#' @param out_direc Directory to place resulting longitudinal dataset into
#' @keywords PSID
#' @export
#' @examples 
#' create_custom_panel(
#'    var_names = c("V534", "V442", "V398"),
#'    in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
#'    out_direc=tempdir()
#' )

create_custom_panel<-function(var_names, in_direc, out_direc){
    #1) Assign directories
        # in_direc<-ifelse(is.null(in_direc),getwd(),in_direc)
        # out_direc<-ifelse(is.null(out_direc),getwd(),out_direc)
        filenames<-list.files(path=in_direc,pattern= c("^FAM.*\\.rds$" ),full.names = T)
        filenames2<-list.files(path=in_direc,pattern= c("^FAM.*\\.rds$" ))
    #2) Error checks
        #a) Subset variables dataframe to lines that contain variable names.
            variables_temp<-as.matrix(data$psid_all)
        #b) Check if each variable is in the dataset. Break if any variable missing.
            missings<-var_names[is.na(match(var_names,variables_temp))]
            if(length(missings)>0){
                message<-paste("The following variable(s) were not found in the PSID family files:",paste(missings,collapse = ", "),sep="\n\n")
                stop(message)
            }
        #c) Exclude redundant variables from following functions. Post warning.
            redundant<-duplicated(var_names)
            if(any(redundant)){
                redundant_name<-var_names[redundant]
                message<-paste("The following variable(s) were requested more than once. Only one copy of each of these variable(s) will be included in the resulting dataset:",paste(redundant_name,collapse = ", "), sep="\n\n")
                var_names<-var_names[!redundant]
                warning(message)
            }
    #3) Temporarily rename each variable to match names of their earliest occurrence.
        temp_names<-var_names
        #a) Identify row and column of each variable
            index<-lapply(temp_names,function(x) which(data$psid_all==x,arr.ind=T))
        #b) Extract rows from index
            row<-sapply(index,function(x) x[1])
        #c) Keep index of current variables
            new_ind<-data$psid_all[row,]
        #d) Rename columms
            names(new_ind)<-gsub('\\D+','', names(new_ind))
        #e) Identify earliest name of each variable
            early_name<-as.vector(apply(new_ind,1,function(x) x[x!=""][1]))

    #4) Create a list of longitudinal datasets with variables selected above
        temp_datal<-list()
        for(j in 1:length(filenames)){
        #a) Load first file and rename to "temp_data"
            temp_data<-readRDS(filenames[j])
        #b) Create vector of variable names
            temp_names<-names(temp_data)
        #c) Only keep columns that match possible variable names
            #i) Identify dataframe year
                temp_year<-gsub('\\D+','', filenames2[j])
            #ii) Identify matchin column in dataset
                column<-match(temp_year,names(new_ind))
            #iii) Find variable names of requested variables as of a given year
                keep_vars<-match(new_ind[,column],temp_names)
                # In case variables already renamed, try above with earliest variable names
                    keep_varsalt<-match(early_name,temp_names)
                    if(sum(is.na(keep_varsalt))<sum(is.na(keep_vars))){
                      keep_vars<-keep_varsalt
                    }
            #iv) Pull those variables and replace no matches with NAs
                keep_vars2<-keep_vars
                keep_vars2[is.na(keep_vars)]<-1
                temp_data2<-temp_data[,keep_vars2]
                temp_data2[,is.na(keep_vars)]<-NA
            #v) rename variables, add year, and append to list
                names(temp_data2)<-var_names
                temp_data2$Year<-as.numeric(temp_year)
                temp_datal[[j]]<-temp_data2
                message(paste(filenames2[j],"Added to Panel"))
        }
    #5) Put above datasets to dataframe format and save
        PSIDPanel<-do.call(rbind,temp_datal)
        dir.create(out_direc,showWarnings = F)
        saveRDS(PSIDPanel,file=paste(out_direc,"/","PSID Panel.rds",sep = ""))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/10find_desc.R
\name{find_description}
\alias{find_description}
\title{Find description of PSID variable}
\usage{
find_description(variables)
}
\arguments{
\item{variables}{Variable names to look up (as individual string or vector of strings)}
}
\description{
Finds the descriptions of selected PSID variables.
}
\examples{
find_description(variables=c("V2","V30"))
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/3rename_fam_vars.R
\name{rename_fam_vars}
\alias{rename_fam_vars}
\title{Rename longitudinal Family File variables}
\usage{
rename_fam_vars(in_direc, out_direc)
}
\arguments{
\item{in_direc}{Directory of PSID .rds files to rename}

\item{out_direc}{Directory for renamed PSID .rds files to be saves to. Warning: If no directory specified, this function will overwrite the Family Files in the current directory.}
}
\description{
Renames all longitudinal variables in every PSID Family File of a given directory, such that variables are labeled with the variable name used when the variable was first made available in the PSID. For example, the "Release Number" variable was first recorded in the PSID dataset in 1968 as variable "V1" but its name in the 1969 family file is "V441". This program changes the "Release Number" variable name to "V1" in 1968 and all subsequent waves.
}
\examples{
rename_fam_vars(
    in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
    out_direc=tempdir()
)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/4rename_ind_vars.R
\name{rename_ind_vars}
\alias{rename_ind_vars}
\title{Renames longitudinal Cross-year Individual variables and saves in long format}
\usage{
rename_ind_vars(in_direc, out_direc, only_long_vars = F, cust_vars = NULL)
}
\arguments{
\item{in_direc}{Directory of PSID Cross-year Individual file .rds file}

\item{out_direc}{Directory for renamed and transformed PSID Cross-year Individual file to be saved to}

\item{only_long_vars}{Keep only longitudinal variables in dataset}

\item{cust_vars}{Custom variables to keep in dataset (as character vector). Output will always include "ER30001", "ER30002", "fam_id_68", and "Year"}
}
\description{
Renames all repeated variables in the Cross-year Individual file so that matching variables across waves have the same name, and transforms the resulting dataset into long format. The longitudinal file does not include rows for respondents who were missing in a given wave, and cross-sectional variables are marked as NA during waves when they were not asked. In addition, the resulting file adds two variables for ease of use: "Year" and "fam_id_68". 

This function may require up to 8gb of RAM, and will likely throw "cannot allocate memory" errors to users with less RAM on their computer. Users with memory issues should implement the "only_long_vars" or "cust_vars" options.
}
\examples{
rename_ind_vars(
   only_long_vars=TRUE, 
   in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
   out_direc=tempdir()
)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/6create_extract.R
\name{create_extract}
\alias{create_extract}
\title{Create subset of common family file variables in long format}
\usage{
create_extract(in_direc, out_direc, num_vars = 500, all_years = F)
}
\arguments{
\item{in_direc}{Directory containing waves of the Family Files in .rds format}

\item{out_direc}{Directory to place export file into}

\item{num_vars}{Number of variables to include in export dataset (default = 500). High variable counts with many waves of data require a significant amount of RAM, and may cause this function to throw errors if a computer's RAM is insufficient}

\item{all_years}{Select most common variables based on all years of the PSID rather than based in the data actually supplied}
}
\description{
Creates an extract dataset in long format consisting of the 500 most frequently reoccurring PSID Family Variables across all supplied waves of the PSID.
}
\examples{
create_extract(
    in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
    out_direc=tempdir(),
    num_vars=25,
)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/1unzip_all_files.R
\name{unzip_all_files}
\alias{unzip_all_files}
\title{Unzip all PSID files}
\usage{
unzip_all_files(in_direc, out_direc)
}
\arguments{
\item{in_direc}{Directory of .zip files to be unzipped}

\item{out_direc}{Directory for unzipped PSID files to be placed}
}
\description{
Unzips all .zip_files files in the specified directory.
}
\examples{
unzip_all_files(
    in_direc=system.file("extdata", "zip_dir", package = "easyPSID"),
    out_direc=tempdir()
)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PSID_Package.Documentation_function.R
\docType{package}
\name{easyPSID-package}
\alias{easyPSID}
\alias{easyPSID-package}
\title{easyPSID: Reading, Formatting, and Organizing the Panel Study of Income Dynamics (PSID)}
\description{
Provides various functions for reading and preparing the Panel Study of Income Dynamics (PSID) for longitudinal analysis, including functions that read the PSID's fixed width format files directly into R, rename all of the PSID's longitudinal variables so that recurring variables have consistent names across years, simplify assembling longitudinal datasets from cross sections of the PSID Family Files, and export the resulting PSID files into file formats common among other statistical programming languages ('SAS', 'STATA', and 'SPSS').
}
\details{
This package is designed for use with the PSID's packaged data Family Files, available at https://simba.isr.umich.edu/. See easyPSID's readme at https://github.com/BrianAronson/easyPSID/blob/master/README.md for a more detailed overview and guide for using this package.
}
\author{
\strong{Maintainer}: Brian Aronson \email{bdaronson@gmail.com}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/5create_custom_panel.R
\name{create_custom_panel}
\alias{create_custom_panel}
\title{Create custom longitudinal (panel) dataset with PSID Family Files}
\usage{
create_custom_panel(var_names, in_direc, out_direc)
}
\arguments{
\item{var_names}{Variable names to include in custom longitudinal dataset (as vector of strings)}

\item{in_direc}{Directory of PSID .rds to use for custom longitudinal dataset}

\item{out_direc}{Directory to place resulting longitudinal dataset into}
}
\description{
Uses the longitudinal PSID Family Files to create a custom longitudinal dataset in long format based on all PSID .rds Family files in a selected directory.

This function can work with data that has been renamed via the rename_fam_vars function or data just converted to .rds format via the convert_to_rds function. It will creates NAs for years when a given variable was not available, and creates a new variable ("Year") to specify the panel of data included in the custom dataset. If a provided variable exists in other waves of the family files under a different name, all waves of that variable will be included in the resulting dataset.

To create a longitudinal family file of the PSID with all variables in the PSID Family Files, it is recommended that one uses the create_extract function instead. However, such a file can be very large when using many waves of the PSID. Users with more than five waves of the PSID Family Files are highly recommended to avoid creating a longitudinal dataset with all unique Family File variables.
}
\examples{
create_custom_panel(
   var_names = c("V534", "V442", "V398"),
   in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
   out_direc=tempdir()
)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/9find_years.R
\name{find_years}
\alias{find_years}
\title{Find years of PSID variable}
\usage{
find_years(variable, var_names = TRUE)
}
\arguments{
\item{variable}{Variable name to look up}

\item{var_names}{Report names for longitudinal PSID variable across years (default=TRUE)}
}
\description{
Finds the years and corresponding variable names for any longitudinal PSID variable in the PSID Family Files and Individual file.
}
\examples{
find_years(variable="V3",var_names=FALSE)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/8find_name.R
\name{find_name}
\alias{find_name}
\title{Lookup new longitudinal variables names}
\usage{
find_name(variable, var_year = TRUE)
}
\arguments{
\item{variable}{Variable to look up}

\item{var_year}{Report year when renamed variable first entered the PSID dataset (default=TRUE)}
}
\description{
Finds the new name of any longitudinal variable in the PSID Family Files or Individual files following implementation of the rename_fam_vars and rename_ind_vars functions.
}
\examples{
find_name(variable="V1244",var_year=FALSE)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/2convert_to_rds.R
\name{convert_to_rds}
\alias{convert_to_rds}
\title{Convert all PSID files from .txt format to .rds format}
\usage{
convert_to_rds(in_direc, out_direc)
}
\arguments{
\item{in_direc}{Directory containing unzipped PSID .txt and .do files}

\item{out_direc}{Directory to place PSID .rds files into}
}
\description{
Converts all PSID fixed width format .txt files in a selected directory into .rds format. This function expects the ASCII .txt file and matching Stata .do definition file contained in each PSID packaged-data .zip download. It does not read Stata .dta files. All files must retain the names they had when first unzipped.
}
\examples{
convert_to_rds(
    in_direc=system.file("extdata","unzip_dir", package = "easyPSID"),
    out_direc=tempdir()
)
}
\keyword{PSID}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/7convert_from_rdata.R
\name{convert_from_rds}
\alias{convert_from_rds}
\title{Export PSID files to another statistical language}
\usage{
convert_from_rds(language, in_direc, out_direc)
}
\arguments{
\item{language}{Language to export PSID .rds files into (options include SPSS, SAS, and STATA)}

\item{in_direc}{Directory of PSID .rds files to export. Note that large files can take a long time to export.}

\item{out_direc}{Directory for exported files to be placed}
}
\description{
Exports all .rds files in the chosen directory into a common file format used by one of three other statistical programming languages (SPSS, SAS, and STATA). Unlike most alternatives, this function retains all variable labels provided by the PSID.
}
\examples{
convert_from_rds(
    language="STATA", 
    in_direc=system.file("extdata","rds_dir", package = "easyPSID"),
    out_direc=tempdir()
)
}
\keyword{PSID}
